"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const pretty_format_1 = __importDefault(require("pretty-format"));
const errors_1 = require("./errors");
const filterNodeByType = (node, type) => node.type === type;
const getNodeByText = (node, text) => {
    try {
        const { Text } = require('@nodegui/react-nodegui');
        const isTextComponent = filterNodeByType(node, Text.toString());
        if (isTextComponent) {
            const textChildren = react_1.default.Children.map(node.props.children, 
            // In some cases child might be undefined or null
            (child) => child !== undefined && child !== null ? child.toString() : '');
            if (textChildren) {
                const textToTest = textChildren.join('');
                return typeof text === 'string'
                    ? text === textToTest
                    : text.test(textToTest);
            }
        }
        return false;
    }
    catch (error) {
        throw errors_1.createLibraryNotSupportedError(error);
    }
};
const getLineEditNodeByPlaceholder = (node, placeholder) => {
    try {
        const { LineEdit } = require('@nodegui/react-nodegui');
        return (filterNodeByType(node, LineEdit.toString()) &&
            (typeof placeholder === 'string'
                ? placeholder === node.props.placeholderText
                : placeholder.test(node.props.placeholderText)));
    }
    catch (error) {
        throw errors_1.createLibraryNotSupportedError(error);
    }
};
const getLineEditNodeByDisplayValue = (node, text) => {
    try {
        const { LineEdit } = require('@nodegui/react-nodegui');
        return (filterNodeByType(node, LineEdit.toString()) &&
            (typeof text === 'string'
                ? text === node.props.text
                : text.test(node.props.text)));
    }
    catch (error) {
        throw errors_1.createLibraryNotSupportedError(error);
    }
};
const getButtonNodeByText = (node, text) => {
    try {
        const { Button } = require('@nodegui/react-nodegui');
        return (filterNodeByType(node, Button.toString()) &&
            (typeof text === 'string'
                ? text === node.props.text
                : text.test(node.props.text)));
    }
    catch (error) {
        throw errors_1.createLibraryNotSupportedError(error);
    }
};
exports.getByType = (instance) => function getByTypeFn(type) {
    try {
        return instance.findByType(type);
    }
    catch (error) {
        throw new errors_1.ErrorWithStack(errors_1.prepareErrorMessage(error), getByTypeFn);
    }
};
exports.getByText = (instance) => function getByTextFn(text) {
    try {
        return instance.find(node => getNodeByText(node, text) || getButtonNodeByText(node, text));
    }
    catch (error) {
        throw new errors_1.ErrorWithStack(errors_1.prepareErrorMessage(error), getByTextFn);
    }
};
exports.getByPlaceholder = (instance) => function getByPlaceholderFn(placeholder) {
    try {
        return instance.find(node => getLineEditNodeByPlaceholder(node, placeholder));
    }
    catch (error) {
        throw new errors_1.ErrorWithStack(errors_1.prepareErrorMessage(error), getByPlaceholderFn);
    }
};
exports.getByDisplayValue = (instance) => function getByDisplayValueFn(placeholder) {
    try {
        return instance.find(node => getLineEditNodeByDisplayValue(node, placeholder));
    }
    catch (error) {
        throw new errors_1.ErrorWithStack(errors_1.prepareErrorMessage(error), getByDisplayValueFn);
    }
};
exports.getByProps = (instance) => function getByPropsFn(props) {
    try {
        return instance.findByProps(props);
    }
    catch (error) {
        throw new errors_1.ErrorWithStack(errors_1.prepareErrorMessage(error), getByPropsFn);
    }
};
exports.getByTestId = (instance) => function getByTestIdFn(testID) {
    try {
        return instance.findByProps({ testID });
    }
    catch (error) {
        throw new errors_1.ErrorWithStack(errors_1.prepareErrorMessage(error), getByTestIdFn);
    }
};
exports.getAllByType = (instance) => function getAllByTypeFn(type) {
    const results = instance.findAllByType(type);
    if (results.length === 0) {
        throw new errors_1.ErrorWithStack('No instances found', getAllByTypeFn);
    }
    return results;
};
exports.getAllByText = (instance) => function getAllByTextFn(text) {
    const results = instance.findAll(node => getNodeByText(node, text) || getButtonNodeByText(node, text));
    if (results.length === 0) {
        throw new errors_1.ErrorWithStack(`No instances found with text: ${String(text)}`, getAllByTextFn);
    }
    return results;
};
exports.getAllByPlaceholder = (instance) => function getAllByPlaceholderFn(placeholder) {
    const results = instance.findAll(node => getLineEditNodeByPlaceholder(node, placeholder));
    if (results.length === 0) {
        throw new errors_1.ErrorWithStack(`No instances found with placeholder: ${String(placeholder)}`, getAllByPlaceholderFn);
    }
    return results;
};
exports.getAllByDisplayValue = (instance) => function getAllByDisplayValueFn(value) {
    const results = instance.findAll(node => getLineEditNodeByDisplayValue(node, value));
    if (results.length === 0) {
        throw new errors_1.ErrorWithStack(`No instances found with display value: ${String(value)}`, getAllByDisplayValueFn);
    }
    return results;
};
exports.getAllByProps = (instance) => function getAllByPropsFn(props) {
    const results = instance.findAllByProps(props);
    if (results.length === 0) {
        throw new errors_1.ErrorWithStack(`No instances found with props:\n${pretty_format_1.default(props)}`, getAllByPropsFn);
    }
    return results;
};
exports.getAllByTestId = (instance) => function getAllByTestIdFn(testID) {
    const results = instance
        .findAllByProps({ testID })
        .filter(element => typeof element.type === 'string');
    if (results.length === 0) {
        throw new errors_1.ErrorWithStack(`No instances found with testID: ${String(testID)}`, getAllByTestIdFn);
    }
    return results;
};
exports.getByAPI = (instance) => ({
    getByTestId: exports.getByTestId(instance),
    getByType: exports.getByType(instance),
    getByText: exports.getByText(instance),
    getByPlaceholder: exports.getByPlaceholder(instance),
    getByDisplayValue: exports.getByDisplayValue(instance),
    getByProps: exports.getByProps(instance),
    getAllByTestId: exports.getAllByTestId(instance),
    getAllByType: exports.getAllByType(instance),
    getAllByText: exports.getAllByText(instance),
    getAllByPlaceholder: exports.getAllByPlaceholder(instance),
    getAllByDisplayValue: exports.getAllByDisplayValue(instance),
    getAllByProps: exports.getAllByProps(instance),
});
//# sourceMappingURL=getByAPI.js.map